﻿Imports Ivi.Visa.Interop
Module FunctionsExample
    '' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
    ''  © Agilent Technologies, Inc. 2013
    ''
    '' You have a royalty-free right to use, modify, reproduce and distribute
    '' the Sample Application Files (and/or any modified version) in any way
    '' you find useful, provided that you agree that Agilent Technologies has no
    '' warranty,  obligations or liability for any Sample Application Files.
    ''
    '' Agilent Technologies provides programming examples for illustration only,
    '' This sample program assumes that you are familiar with the programming
    '' language being demonstrated and the tools used to create and debug
    '' procedures. Agilent Technologies support engineers can help explain the
    '' functionality of Agilent Technologies software components and associated
    '' commands, but they will not modify these samples to provide added
    '' functionality or construct procedures to meet your specific needs.
    '' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
    Sub Main()
        'This example program illustrates how to connect to an instrument
        'Gives an example setup for all major function configurations (DCV, DCI, Ohms2W, Ohms 4W, ACV, ACI, Temp, Freq). 
        'Gets a single reading from the DMM for each function and reports the result. 
        'Ohms 4W shows an autorange setup while all other functions show a direct range setup.



        'For more information on getting started using VISA COM I/O operations see the app note located at:
        'http:'cp.literature.agilent.com/litweb/pdf/5989-6338EN.pdf
        Dim rm As Ivi.Visa.Interop.ResourceManager = New Ivi.Visa.Interop.ResourceManager()   'Open up a new resource manager
        Dim myDmm As Ivi.Visa.Interop.FormattedIO488 = New Ivi.Visa.Interop.FormattedIO488()  'Open a new Formatted IO 488 session 

        Try

            'Dim DutAddr as String = "GPIB0::22"  'String for GPIB
            'Dim DutAddr as String = "TCPIP0::169.254.4.61"   'Example string for LAN
            'Dim DutAddr as String = "USB0::0x0957::0x1A07::MY53000101"  'Example string for USB
            'Dim DutAddr As String = "USB0::0x0957::0x1C07::US00000069::0::INSTR"  'String for GPIB
            Dim DutAddr As String = "TCPIP0::156.140.92.16"   'Example string for LAN

            myDmm.IO = rm.Open(DutAddr, AccessMode.NO_LOCK, 2000, "")  'Open up a handle to the DMM with a 2 second timeout
            myDmm.IO.Timeout = 3000  'You can also set your timeout by doing this command, sets to 3 seconds

            'First start off with a reset state
            myDmm.IO.Clear()  'Send a device clear first to stop any measurements in process
            myDmm.WriteString("*RST")  'Reset the device
            myDmm.WriteString("*IDN?")  'Get the IDN string                
            Dim IDN As String = myDmm.ReadString()
            Console.WriteLine(IDN)  'report the DMM's identity

            'Configure for DCV 100V range, 100uV resolution
            myDmm.WriteString("CONF:VOLT:DC 100, 0.0001")
            myDmm.WriteString("READ?")
            Dim DCVResult As String = myDmm.ReadString()
            Console.WriteLine("DCV Reading = " + DCVResult)  'report the DCV reading
            CheckDMMError(myDmm)  'Check if the DMM has any errors

            'Configure for ACV 100V range, to read a 100 Hz signal
            myDmm.WriteString("CONF:VOLT:AC 100, 0.0001")
            myDmm.WriteString("VOLT:AC:BAND 20") 'Choose the band that is lower than your input frequency, Bands are 3 Hz|20 Hz|200 Hz
            myDmm.WriteString("READ?")
            Dim ACVResult As String = myDmm.ReadString()
            Console.WriteLine("ACV Reading = " + ACVResult)  'report the ACV reading
            CheckDMMError(myDmm)  'Check if the DMM has any errors

            'Configure for DCI 10A range, 100uA resolution
            myDmm.WriteString("CONF:CURR:DC 10, 0.0001")
            myDmm.WriteString("READ?")
            Dim DCIResult As String = myDmm.ReadString()
            Console.WriteLine("DCI Reading = " + DCIResult)  'report the DCI reading
            CheckDMMError(myDmm)  'Check if the DMM has any errors

            'Configure for ACI 10mA range, to read a 100 KHz signal
            myDmm.WriteString("CONF:CURR:AC .01, 0.0001")
            myDmm.WriteString("VOLT:AC:BAND 200") 'Choose the band that is lower than your input frequency, Bands are 3 Hz|20 Hz|200 Hz
            myDmm.WriteString("READ?")
            Dim ACIResult As String = myDmm.ReadString()
            Console.WriteLine("ACI Reading = " + ACIResult)  'report the ACV reading
            CheckDMMError(myDmm)  'Check if the DMM has any errors

            'Configure for OHM 2 wire 100 Ohm range, 100uOhm resolution
            myDmm.WriteString("CONF:RES 100, 0.0001")
            myDmm.WriteString("READ?")
            Dim Res2WResult As String = myDmm.ReadString()
            Console.WriteLine("2 Wire Resistance Reading = " + Res2WResult)  'report the 2W resistance reading
            CheckDMMError(myDmm)  'Check if the DMM has any errors

            'Configure for OHM 4 wire Autoranged
            myDmm.WriteString("CONF:FRES")
            myDmm.WriteString("READ?")
            Dim Res4WResult As String = myDmm.ReadString()
            Console.WriteLine("4 Wire Resistance Reading = " + Res4WResult)  'report the 2W resistance reading
            CheckDMMError(myDmm)  'Check if the DMM has any errors


            'Configure for Frequency 100V, 1Khz signal, maximum resolution
            myDmm.WriteString("CONF:FREQ 200, MAX")  'Sets the bandwidth range to 200 (highest BW range) which will measure a 1Khz signal
            myDmm.WriteString("FREQ:VOLT:RANG  100")
            myDmm.WriteString("READ?")
            Dim FreqResult As String = myDmm.ReadString()
            Console.WriteLine("Freq Reading = " + FreqResult)  'report the 2W resistance reading
            CheckDMMError(myDmm)  'Check if the DMM has any errors

            'Configure for Temperature reading with a 5K thermistor
            myDmm.WriteString("CONF:Temp THER, 5000")
            myDmm.WriteString("READ?")
            Dim TempResult As String = myDmm.ReadString()
            Console.WriteLine("Temperature Reading = " + TempResult)  'report the Temperature reading
            CheckDMMError(myDmm)  'Check if the DMM has any errors

        Catch e As Exception

            Console.WriteLine("Error occured: " + e.Message)

        Finally

            'Close out your resources
            Try
                myDmm.IO.Close()
            Catch ex As Exception

            End Try
            Try
                System.Runtime.InteropServices.Marshal.ReleaseComObject(myDmm)
            Catch ex As Exception

            End Try

            Try
                System.Runtime.InteropServices.Marshal.ReleaseComObject(rm)
            Catch ex As Exception

            End Try
            Console.WriteLine("Press any key to continue...")
            Console.ReadKey()

        End Try

    End Sub
    Sub CheckDMMError(myDmm As FormattedIO488)
        myDmm.WriteString("SYST:ERR?")
        Dim errStr As String = myDmm.ReadString()

        If (errStr.Contains("No error")) Then 'If no error, then return
            Return
            'If there is an error, read out all of the errors and return them in an exception
        Else
            Dim errStr2 As String = ""
            Do
                myDmm.WriteString("SYST:ERR?")
                errStr2 = myDmm.ReadString()
                If (Not errStr2.Contains("No error")) Then
                    errStr = errStr + "\n" + errStr2
                End If
            Loop While (Not errStr2.Contains("No error"))
            Throw New Exception("Exception: Encountered system error(s)\n" + errStr)

        End If

    End Sub

End Module
