﻿Imports Ivi.Visa.Interop
Module DMMMath
    '' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
    ''  © Agilent Technologies, Inc. 2013
    ''
    '' You have a royalty-free right to use, modify, reproduce and distribute
    '' the Sample Application Files (and/or any modified version) in any way
    '' you find useful, provided that you agree that Agilent Technologies has no
    '' warranty,  obligations or liability for any Sample Application Files.
    ''
    '' Agilent Technologies provides programming examples for illustration only,
    '' This sample program assumes that you are familiar with the programming
    '' language being demonstrated and the tools used to create and debug
    '' procedures. Agilent Technologies support engineers can help explain the
    '' functionality of Agilent Technologies software components and associated
    '' commands, but they will not modify these samples to provide added
    '' functionality or construct procedures to meet your specific needs.
    '' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
    Sub Main()
        'Uses the CALC subsystem. This system does internal calculations on the data set include statistics and limit checking. 
        '1)Take 20 readings and turn on internal limits. Set limit and check the questions status byte if a reading was outside of limits.
        '2)Then perform math functions including mean, max, min, standard deviation, and peak to peak. The max and min reading will show the readings that were outside of the limits


        'For more information on getting started using VISA COM I/O operations see the app note located at:
        'http:'cp.literature.agilent.com/litweb/pdf/5989-6338EN.pdf
        Dim rm As Ivi.Visa.Interop.ResourceManager = New Ivi.Visa.Interop.ResourceManager()   'Open up a new resource manager
        Dim myDmm As Ivi.Visa.Interop.FormattedIO488 = New Ivi.Visa.Interop.FormattedIO488()  'Open a new Formatted IO 488 session 

        Try

            'Dim DutAddr as String = "GPIB0::22"  'String for GPIB
            'Dim DutAddr as String = "TCPIP0::169.254.4.61"   'Example string for LAN
            'Dim DutAddr as String = "USB0::0x0957::0x1A07::MY53000101"  'Example string for USB
            'Dim DutAddr As String = "USB0::0x0957::0x1C07::US00000069::0::INSTR"  'String for GPIB
            Dim DutAddr As String = "TCPIP0::156.140.92.16"   'Example string for LAN

            myDmm.IO = rm.Open(DutAddr, AccessMode.NO_LOCK, 2000, "")  'Open up a handle to the DMM with a 2 second timeout
            myDmm.IO.Timeout = 3000  'You can also set your timeout by doing this command, sets to 3 seconds

            'First start off with a reset state
            myDmm.IO.Clear()  'Send a device clear first to stop any measurements in process
            myDmm.WriteString("*RST")  'Reset the device
            myDmm.WriteString("*IDN?")  'Get the IDN string                
            Dim IDN As String = myDmm.ReadString()
            Console.WriteLine(IDN)  'report the DMM's identity
            

            myDmm.WriteString("CONF:VOLT:DC 1, 1e-4")  'Configure a semi-fast DCV measurement
            myDmm.WriteString("SAMP:COUN 20")  'Get 20 Readings per trigger

            'Clear all calculations before we start
            myDmm.WriteString("CALC:AVER:CLE")
            myDmm.WriteString("CALC:AVER:STAT ON")  'Turn on Stat calculations for future readings

            'Set the limits and turn on internal limit checking
            myDmm.WriteString("CALC:LIM:LOW -1e-3")  'Set lower limit to -1mV
            myDmm.WriteString("CALC:LIM:UPP 1e-3")  'Set upper limit to -1mV

            myDmm.WriteString("CALC:LIM:STAT ON")  'Turn on Limits for future readings

            'Get the readings
            myDmm.WriteString("READ?")  'Trigger the DMM to start the readings, can also use INIT command
            Dim DCVResult As Object() = myDmm.ReadList(IEEEASCIIType.ASCIIType_Any)  ' Get the readings in an array

            For Each rdg As Double In DCVResult ' Write out each result to the console
                Console.WriteLine(rdg.ToString())
            Next

            'Read out the Questionable status register to determine programmatically if there was a limit failure
            myDmm.WriteString("STAT:QUES?")
            Dim Status As Integer = CType(myDmm.ReadNumber(IEEEASCIIType.ASCIIType_I4, True), Integer) ' read the DMM and cast the results as integer
            'if Bit 12 from status register is set then the upper limit failed
            Dim uppLimFail As Integer = 4096 & Status
            'If bit 11 from status register is set then the lower limit failed.
            Dim lowLimFail As Integer = 2048 & Status

            If uppLimFail = 4096 Then
                Console.WriteLine("A reading failed Upper limit!")
            End If
            If lowLimFail = 2048 Then
                Console.WriteLine("A reading failed Lower limit!")
            End If


            'Let's get some statistics from the measurements that we just took
                 
            myDmm.WriteString("CALC:AVER:MAX?")  'Get maximum from the readings
            Dim max As Double = CType(myDmm.ReadNumber(IEEEASCIIType.ASCIIType_R4, True), Double) 'Get reading and cast result as double           
            CheckDMMError(myDmm)  'Check DMM for error
            Console.WriteLine("Max = " + max.ToString())

            myDmm.WriteString("CALC:AVER:MIN?")  'Get minimum from the readings
            Dim min As Double = myDmm.ReadNumber(IEEEASCIIType.ASCIIType_R4, True)
            CheckDMMError(myDmm)  'Check DMM for error
            Console.WriteLine("Min = " + min.ToString())


            'Let's do some internal math on the readings
            myDmm.WriteString("CALC:AVER:AVER?")  'Get average from the readings
            Dim mean As Double = myDmm.ReadNumber(IEEEASCIIType.ASCIIType_R4, True)
            CheckDMMError(myDmm)  'Check DMM for error
            Console.WriteLine("Mean = " + mean.ToString())

            myDmm.WriteString("CALC:AVER:SDEV?")  'Get standard dev from the readings
            Dim sdev As Double = myDmm.ReadNumber(IEEEASCIIType.ASCIIType_R4, True)
            CheckDMMError(myDmm)  'Check DMM for error
            Console.WriteLine("Sdev = " + sdev.ToString())



        Catch e As Exception

            Console.WriteLine("Error occured: " + e.Message)

        Finally

            'Close out your resources
            Try
                myDmm.IO.Close()
            Catch ex As Exception

            End Try
            Try
                System.Runtime.InteropServices.Marshal.ReleaseComObject(myDmm)
            Catch ex As Exception

            End Try

            Try
                System.Runtime.InteropServices.Marshal.ReleaseComObject(rm)
            Catch ex As Exception

            End Try
            Console.WriteLine("Press any key to continue...")
            Console.ReadKey()

        End Try
    End Sub
    Sub CheckDMMError(myDmm As FormattedIO488)
        myDmm.WriteString("SYST:ERR?")
        Dim errStr As String = myDmm.ReadString()

        If (errStr.Contains("No error")) Then 'If no error, then return
            Return
            'If there is an error, read out all of the errors and return them in an exception
        Else
            Dim errStr2 As String = ""
            Do
                myDmm.WriteString("SYST:ERR?")
                errStr2 = myDmm.ReadString()
                If (Not errStr2.Contains("No error")) Then
                    errStr = errStr + "\n" + errStr2
                End If
            Loop While (Not errStr2.Contains("No error"))
            Throw New Exception("Exception: Encountered system error(s)\n" + errStr)

        End If

    End Sub

End Module
